@@ -3,12 +3,13 @@ from __future__ import absolute_import
 from __future__ import division
 
 import json
-import sys
+import platform
+import uuid
 
 from compas.data.exceptions import DecoderError
 
 # We don't do this from `compas.IPY` to avoid circular imports
-if 'ironpython' in sys.version.lower():
+if "ironpython" == platform.python_implementation().lower():
     try:
         from System.Collections.Generic import IDictionary
     except:  # noqa: E722
@@ -28,7 +29,7 @@ def cls_from_dtype(dtype):
 
     Returns
     -------
-    :class:`compas.base.Base`
+    :class:`~compas.base.Base`
 
     Raises
     ------
@@ -40,7 +41,7 @@ def cls_from_dtype(dtype):
         If the module doesn't contain the specified data type.
 
     """
-    mod_name, attr_name = dtype.split('/')
+    mod_name, attr_name = dtype.split("/")
     module = __import__(mod_name, fromlist=[attr_name])
     return getattr(module, attr_name)
 
@@ -52,7 +53,7 @@ class DataEncoder(json.JSONEncoder):
 
     * Numpy objects to their Python equivalents;
     * iterables to lists; and
-    * :class:`compas.data.Data` objects,
+    * :class:`~compas.data.Data` objects,
       such as geometric primitives and shapes, data structures, robots, ...,
       to a dict with the following structure: ``{'dtype': o.dtype, 'value': o.data}``
 
@@ -97,18 +98,19 @@ class DataEncoder(json.JSONEncoder):
             The serialized object.
 
         """
-        if hasattr(o, 'to_data'):
+        if hasattr(o, "to_data"):
             value = o.to_data()
-            if hasattr(o, 'dtype'):
+            if hasattr(o, "dtype"):
                 dtype = o.dtype
             else:
-                dtype = '{}/{}'.format('.'.join(o.__class__.__module__.split('.')[:-1]), o.__class__.__name__)
-            return {
-                'dtype': dtype,
-                'value': value
-            }
+                dtype = "{}/{}".format(
+                    ".".join(o.__class__.__module__.split(".")[:-1]),
+                    o.__class__.__name__,
+                )
 
-        if hasattr(o, '__next__'):
+            return {"dtype": dtype, "value": value, "guid": str(o.guid)}
+
+        if hasattr(o, "__next__"):
             return list(o)
 
         try:
@@ -118,9 +120,22 @@ class DataEncoder(json.JSONEncoder):
         else:
             if isinstance(o, np.ndarray):
                 return o.tolist()
-            if isinstance(o, (np.int_, np.intc, np.intp, np.int8,
-                              np.int16, np.int32, np.int64, np.uint8,
-                              np.uint16, np.uint32, np.uint64)):
+            if isinstance(
+                o,
+                (
+                    np.int_,
+                    np.intc,
+                    np.intp,
+                    np.int8,
+                    np.int16,
+                    np.int32,
+                    np.int64,
+                    np.uint8,
+                    np.uint16,
+                    np.uint32,
+                    np.uint64,
+                ),
+            ):
                 return int(o)
             if isinstance(o, (np.float_, np.float16, np.float32, np.float64)):
                 return float(o)
@@ -136,7 +151,7 @@ class DataDecoder(json.JSONDecoder):
     """Data decoder for custom JSON serialization with support for COMPAS data structures and geometric primitives.
 
     The decoder hooks into the JSON deserialisation process
-    to reconstruct :class:`compas.data.Data` objects,
+    to reconstruct :class:`~compas.data.Data` objects,
     such as geometric primitives and shapes, data structures, robots, ...,
     from the serialized data when possible.
 
@@ -185,25 +200,39 @@ class DataDecoder(json.JSONDecoder):
             A (reconstructed), deserialized object.
 
         """
-        if 'dtype' not in o:
+        if "dtype" not in o:
             return o
 
         try:
-            cls = cls_from_dtype(o['dtype'])
+            cls = cls_from_dtype(o["dtype"])
 
         except ValueError:
-            raise DecoderError("The data type of the object should be in the following format: '{}/{}'".format(o.__class__.__module__, o.__class__.__name__))
+            raise DecoderError(
+                "The data type of the object should be in the following format: '{}/{}'".format(
+                    o.__class__.__module__, o.__class__.__name__
+                )
+            )
 
         except ImportError:
-            raise DecoderError("The module of the data type can't be found: {}.".format(o['dtype']))
+            raise DecoderError(
+                "The module of the data type can't be found: {}.".format(o["dtype"])
+            )
 
         except AttributeError:
-            raise DecoderError("The data type can't be found in the specified module: {}.".format(o['dtype']))
+            raise DecoderError(
+                "The data type can't be found in the specified module: {}.".format(
+                    o["dtype"]
+                )
+            )
 
-        obj_value = o['value']
+        obj_value = o["value"]
 
         # Kick-off from_data from a rebuilt Python dictionary instead of the C# data type
         if IDictionary and isinstance(o, IDictionary[str, object]):
             obj_value = {key: obj_value[key] for key in obj_value.Keys}
 
-        return cls.from_data(obj_value)
+        obj = cls.from_data(obj_value)
+        if "guid" in o:
+            obj._guid = uuid.UUID(o["guid"])
+
+        return obj
